# callbacks.py
# Abstract interfaces to communicate progress on tasks.
#
# Copyright (C) 2014-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
from __future__ import unicode_literals
import dnf.yum.rpmtrans

import dnf.transaction
import libdnf.transaction

PKG_DOWNGRADED = libdnf.transaction.TransactionItemAction_DOWNGRADED
PKG_DOWNGRADE = libdnf.transaction.TransactionItemAction_DOWNGRADE
PKG_INSTALL = libdnf.transaction.TransactionItemAction_INSTALL
PKG_OBSOLETED = libdnf.transaction.TransactionItemAction_OBSOLETED
PKG_OBSOLETE = libdnf.transaction.TransactionItemAction_OBSOLETE
PKG_REINSTALLED = libdnf.transaction.TransactionItemAction_REINSTALLED
PKG_REINSTALL = libdnf.transaction.TransactionItemAction_REINSTALL
PKG_REMOVE = libdnf.transaction.TransactionItemAction_REMOVE
PKG_UPGRADE = libdnf.transaction.TransactionItemAction_UPGRADE
PKG_UPGRADED = libdnf.transaction.TransactionItemAction_UPGRADED

PKG_CLEANUP = 101
PKG_SCRIPTLET = 103
PKG_VERIFY = 102
TRANS_POST = 202
TRANS_PREPARATION = 201
PKG_ERASE = PKG_REMOVE

STATUS_DRPM = 4    # :api
STATUS_MIRROR = 3  # :api
STATUS_FAILED = 1 # :api
STATUS_ALREADY_EXISTS = 2 # :api
STATUS_OK = None # :api

class Payload(object):
    # :api
    def __init__(self, prgs):
        self.progress = prgs

    def __str__(self):
        """Nice, human-readable representation. :api"""
        pass
    @property
    def download_size(self):
        """Total size of the download. :api"""
        pass

class KeyImport(object):
    def _confirm_ctl(self, id, userid, fingerprint, url, timestamp):
        """Ask the user if the key should be imported."""
        return False

class DownloadProgress(object):
    # :api
    def start(self, total_files, total_size, total_drpms=0):
        """Start new progress metering. :api

        `total_files` the number of files that will be downloaded,
        `total_size` total size of all files.

        """
        pass
    def progress(self, payload, done):
        """Update the progress display. :api

        `payload` is the payload this call reports progress for, `done` is how
        many bytes of this payload are already downloaded.

        """
        pass
    def end(self, payload, status, download_msg):
        """Communicate the information that `payload` has finished downloading.

        :api, `status` is a constant denoting the type of outcome, `err_msg` is an
        error message in case the outcome was an error.

        """
        pass
    def message(self, download_msg):
        pass

class Depsolve(object):
    def end(self):
        pass
    def start(self):
        pass
    def pkg_added(self, pkg, mode):
        pass
class NullDownloadProgress(DownloadProgress):
    pass
TransactionProgress = dnf.yum.rpmtrans.TransactionDisplay  # :api