# match_counter.py
# Implements class MatchCounter.
#
# Copyright (C) 2012-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from functools import reduce

def _canonize_string_set(sset, length):
    current = len(sset)
    return [''] * (length - current) + sorted(sset)

WEIGHTS = {'name'		: 7,
    'summary'		: 4,
    'description'	: 2,
    'url'		: 1,
    }

class MatchCounter(dict):
    @staticmethod
    def _eval_weights(pkg, matches):
        # how much is each match worth and return their sum:
        def weight(match):
            key = match[0]
            needle = match[1]
            haystack = getattr(pkg, key)
            if key == "name" and haystack == needle:
                return 2 * WEIGHTS[key]
            return WEIGHTS[key]

        return sum(map(weight, matches))
    
    def _max_needles(self):
        """Return the max count of needles of all packages."""
        if self:
            m = 0
            for pkg in self:
                m = max(m,len(self.matched_needles(pkg)))
            return m
        return 0

    def _key_func(self):
        def get_key(pkg):
            return (- self._eval_weights(pkg, self[pkg]),pkg.name,)
        return get_key

    def dump(self):
        for p in self:
            print('%s\t%s' % (p, self[p]))
        
    def add(self, pkg, key, needle):
        self.setdefault(pkg, []).append((key, needle))

    def matched_haystacks(self, pkg):
        t = set()
        for m in self[pkg]:
            t.add(getattr(pkg, m[0]))
        return t

    def matched_needles(self, pkg):
        t = set()
        for m in self[pkg]:
            t.add(m[1])
        return t
    
    def matched_keys(self, pkg):
        result = []
        for i in self[pkg]:
            if i[0] not in result:
                result.append(i[0])
        return result
    
    def total(self):
        return reduce(lambda total, pkg: total + len(self[pkg]), self, 0)

    def sorted(self, reverse=False, limit_to=None):
        if limit_to:
            keys = limit_to
        else:
            keys = self.keys()
        return sorted(keys, key=self._key_func())
